C#######################################################################
C
C MCTDH-module DENSITY
C
C Calculates the reduced density matrices, their inverse matrices and
C  their eigenvalues and eigenvectors.
C
C density: passing routine
C denmat: calculates the reduced density matrices
C matinv: inverts a matrix
C
C#######################################################################

C#######################################################################
C DENSITY
C
C passes wavefunction coefficients and relevant part of density matix
C  arrays for calculation of reduced density matrices for each mode.
C
C all passed variables are standard arrays, see documentation
C
C V6.3 MB
C#######################################################################

!>  this subroutine is part of the heidelberg mctdh package
!>  please see the source code for details
      subroutine density(Inv_rho_jk,rho_jk,dicht3,dicht4,workr)
      USE moduleinputvariables,ONLY:Morb 
      implicit none
      real*8 dicht3(Morb,2),epsrel,norm,trace,workr(3*MOrb)
      REAL*8, PARAMETER     :: epsmat = 1.D-8 !KS
      complex*16 Inv_rho_jk(Morb,Morb),rho_jk(Morb,Morb),
     +           dicht4(Morb,Morb)


C-----------------------------------------------------------------------
C compute norm and take a relative regularization parameter
C-----------------------------------------------------------------------
      norm=0.0d0
      call trhxz(rho_jk,trace,Morb)
      norm=norm+trace
      epsrel=norm*epsmat

C-----------------------------------------------------------------------
C regularize and invert density matrices for each mode and state
C-----------------------------------------------------------------------
      call denmat(Inv_rho_jk,rho_jk,
     +     dicht3,dicht4,Morb,
     +     workr,epsrel)

      return
      end
      
C#######################################################################
C DENMAT
C
C called by density
C calls to matinv
C
C Calculates the reduced density matrix etc. for a single mode. For 
C  natural orbital propagation, the eigenvalues and inverse 
C  density matrices are taken directly from the diagonal of dicht2,
C  otherwise dicht2 is diagonalised.
C
C workc: complex workspace needed by subroutine matinv
C for the mode passed by subroutine density:
C  dicht1: inverse density matrix
C  dicht2: reduced density matrix
C  dicht3: eigenvalues and regularised eigenvalues of dicht2
C  dicht4: eigenvectors of dicht2
C  dim: number of single particle functions
C
C  KS 2009 adopted and modified from MCTDH 
C#######################################################################

!>  this subroutine is part of the heidelberg mctdh package
!>  please see the source code for details
      subroutine denmat(dicht1,dicht2,dicht3,dicht4,dim,
     +                  workr,epsrel)

      implicit none
      
      integer e,e1,dim
      complex*16 dicht1(dim,dim),dicht2(dim,dim),
     +   dicht4(dim,dim)
      real*8 dicht3(dim,2),epsrel,workr(3*dim)

C-----------------------------------------------------------------------
C calculate regularized invers density matrix
C Notice that negative of dicht2 is inverted, so that eigenvalues are
C ordered with largest at dicht3(1)
C-----------------------------------------------------------------------
      
         do e=1,dim
            do e1=1,dim
               dicht4(e1,e)=-dicht2(e1,e)
            enddo
         enddo
         call matinv(dicht1,dicht3,dicht4,dim,epsrel,workr)

      return
      end

C#######################################################################
C
C MATINV
C
C called by denmat
C calls to zheev 
C
C calls diagonalisation routines for density matrix and from the
C  eigenvalues (regularised) and eigenvectors so obtained calculates 
C  the inverse density matrix
C 
C invers: returned as inverted matrix, used as scratch array.
C eval:   eigenvalues of passed matrix
C evek:   eigenvectors of passed matrix; on input: Matrix to be inverted.
C dim:    dimensions of matrices e.g. invers(dim,dim)
C work:   complex workspace         
C
C KS adopted  and modified from MCTDH
C#######################################################################
  
!>  this subroutine is part of the heidelberg mctdh package
!>  please see the source code for details
      subroutine matinv(invers,eval,evek,dim,epsrel,work)

      implicit none


      integer       error,e,e1,e2,dim
      complex*16    invers(dim,dim),evek(dim,dim)
      real*8        eval(dim,2),epsrel,work(3*dim)

C-----------------------------------------------------------------------
C diagonalize matrix
C-----------------------------------------------------------------------
      e2 = dim*dim
      call zheev('V','U',dim,evek,dim,eval,invers,e2,work,error)
      if (error .ne. 0) then
!         routine = 'matinv'
!         message = 'Cannot diagonalize density matrix'
         write(*,*)'matinv'
         write(*,*)'Cannot diagonalize density matrix'
!         call errormsg
      endif

C-----------------------------------------------------------------------
C take negative of eigenvalue (due to re-ordering) and regularize.
C The if-statement is introduced to avoid an underflow.
C-----------------------------------------------------------------------
      do e=1,dim
         eval(e,1)=-eval(e,1)
         if( eval(e,1) .lt. 64.d0*epsrel) then
            eval(e,2)=eval(e,1)+epsrel*exp(-eval(e,1)/epsrel)
         else
            eval(e,2)=eval(e,1)
         endif
      enddo

C-----------------------------------------------------------------------
C calculate regularized inverse matrix
C-----------------------------------------------------------------------
      call zeromxz(invers,dim,dim)
      do e=1,dim
        do e1=1,dim
          do e2=1,dim
             invers(e1,e)=invers(e1,e)+
     +            dconjg(evek(e,e2))*evek(e1,e2)/eval(e2,2)
          enddo
        enddo
        invers(e,e)=dble(invers(e,e))
      enddo

      return
      end


